@@ -32,240 +32,155 @@
 //-----------------------------------------------------------------------------
 #include "chuck_parse.h"
 #include "chuck_errmsg.h"
-#include <string.h>
-
-#ifdef __ANDROID__
+#include "util_string.h"
 #include "util_platforms.h"
+
+#include <string.h>
+#include <sys/stat.h>
+#if defined(__PLATFORM_WINDOWS__)
+  #include "dirent_win32.h"
 #endif
 
-#ifdef __PLATFORM_WIN32__
-#ifndef __CHUNREAL_ENGINE__
-  #include <windows.h> // for win32_tmpfile()
-#else
-  // 1.5.0.0 (ge) | #chunreal
-  // unreal engine on windows disallows including windows.h
-  #include "Windows/MinWindows.h"
-#endif // #ifndef __CHUNREAL_ENGINE__
-#endif // #ifdef __PLATFORM_WIN32__
-
+#include <string>
 using namespace std;
 
-// max path len
-static const t_CKUINT MAX_FILENAME_LEN = 2048;
 // global
-static char g_filename[MAX_FILENAME_LEN+1] = "";
+static std::string g_filename;
+// file to parse
+static FILE * g_fd2parse = NULL;
+// whether to auto close
+static t_CKBOOL g_fd2autoClose = FALSE;
+// separate file to close
+static FILE * g_fd2close = NULL;
+// clean up function
+static void fd2parse_cleanup();
 
-// external
+// external from lexer
 extern "C" {
-    extern FILE *yyin;
+    extern FILE * yyin;
 }
 
 
 
 
 //-----------------------------------------------------------------------------
-// name: open_cat_ck()
-// desc: ...
+// name: cleanup_AST()
+// desc: clean up the abstract syntax tree
 //-----------------------------------------------------------------------------
-FILE * open_cat_ck( c_str fname )
+static void cleanup_AST()
 {
-#ifdef __ANDROID__
-    if( strncmp(fname, "jar:", strlen("jar:")) == 0 )
-    {
-        int fd = 0;
-        if( !ChuckAndroid::copyJARURLFileToTemporary(fname, &fd) )
-        {
-            EM_error2( 0, "unable to download from JAR URL: %s", fname );
-            return NULL;
-        }
-        return fdopen(fd, "rb");
-    }
-#endif // __ANDROID__
+    // check
+    if( !g_program ) return;
 
-    FILE * fd = fopen(fname, "rb");
-    if( !fd )
-    {
-        if( !strstr( fname, ".ck" ) && !strstr( fname, ".CK" ) )
-        {
-            strcat( fname, ".ck" );
-            fd = fopen( fname, "rb" );
-        }
-    }
-    return fd;
+    // log
+    EM_log( CK_LOG_FINE, "cleaning up abstract syntax tree..." );
+    // do it
+    delete_program( g_program );
+    // reset to NULL
+    g_program = NULL;
 }
 
 
 
 
-//-----------------------------------------------------------------------------
-// name: win32_tmpfile()
-// desc: replacement for broken tmpfile() on Windows Vista + 7
-//-----------------------------------------------------------------------------
-#ifdef __PLATFORM_WIN32__
-FILE *win32_tmpfile()
-{
-    char tmp_path[MAX_PATH];
-    char file_path[MAX_PATH];
-    FILE * file = NULL;
-
-#ifndef __CHUNREAL_ENGINE__
-    if( GetTempPath(256, tmp_path) == 0 )
-#else
-    // 1.5.0.0 (ge) | #chunreal explicit call ASCII version
-    if( GetTempPathA(256, tmp_path) == 0 )
-#endif
-        return NULL;
-
-#ifndef __CHUNREAL_ENGINE__
-    if( GetTempFileName(tmp_path, "mA", 0, file_path) == 0 )
-#else
-    // 1.5.0.0 (ge) | #chunreal explicit call ASCII version
-    if( GetTempFileNameA(tmp_path, "mA", 0, file_path) == 0 )
-#endif
-        return NULL;
-
-    file = fopen( file_path, "wb+D" );
-
-    return file;
-}
-#endif // #ifdef __PLATFORM_WIN32__
-
-
-
-
-//-----------------------------------------------------------------------------
-// name: android_tmpfile()
-// desc: replacement for broken tmpfile() on Android
-//-----------------------------------------------------------------------------
-#ifdef __ANDROID__
-
-FILE * android_tmpfile()
-{
-    return ChuckAndroid::getTemporaryFile();
-}
-
-#endif
-
-
-
-
 //-----------------------------------------------------------------------------
 // name: chuck_parse()
-// desc: ...
+// desc: INPUT: chuck code (either from file or actual code) to be parsed
+//       OUTPUT: abstract syntax tree representation of the code
 //-----------------------------------------------------------------------------
-t_CKBOOL chuck_parse( c_constr fname, FILE * fd, c_constr code )
+t_CKBOOL chuck_parse( const std::string & fname, const std::string & codeLiteral )
 {
-    t_CKBOOL clo = FALSE;
+    // return value
     t_CKBOOL ret = FALSE;
+    // file descriptor | normally NULL unless g_fd2parse is set
+    FILE * fd = g_fd2parse;
+    // our own lexer/parser buffer
+    YY_BUFFER_STATE yyCodeBuffer = NULL;
+    // where the code is coming from
+    CompileFileSource source;
 
-    // sanity check
-    if( fd && code )
+    // check for conflict
+    if( fd && codeLiteral != "" )
     {
-        CK_FPRINTF_STDERR( "[chuck](via parser): (internal) both fd and code specified!\n" );
-        return FALSE;
+        CK_FPRINTF_STDERR( "[chuck](parser): (internal) code and FILE descriptor both present!\n" );
+        CK_FPRINTF_STDERR( "[chuck](parser):  |- ignoring FILE descriptor...\n" );
     }
 
-    // prepare code
-    if( code )
+    // copy it
+    g_filename = fname;
+
+    // if actual code was passed in
+    if( codeLiteral != "" )
     {
-        // !
-        assert( fd == NULL );
-        // generate temp file
-#ifdef __PLATFORM_WIN32__
-        fd = win32_tmpfile();
-#elif defined (__ANDROID__)
-        fd = android_tmpfile();
-#else
-        fd = tmpfile();
-#endif
-        // on some systems, tmpfile() can return NULL
-        if( !fd ) { EM_error2( 0, "unable to create temp file" ); return FALSE; }
-        // flag it to close
-        clo = TRUE;
-        // write
-        fwrite( code, sizeof(char), strlen(code), fd );
-    }
-
-    /*
-    // use code from memory buffer if its available
-    if( code )
-    {
-        // copy name
-        strcpy( g_filename, fname );
-        // reset
-        if( EM_reset( g_filename, NULL ) == FALSE ) goto cleanup;
-
-        // TODO: clean g_program
-        g_program = NULL;
-
-        // clean
+        // clean lexer
         yyrestart( NULL );
-
         // load string (yy_scan_string will copy the C string)
-        YY_BUFFER_STATE ybs = yy_scan_string( code );
-        if( !ybs ) goto cleanup;
-
-        // parse
-        if( !( yyparse() == 0 ) ) goto cleanup;
-
-        // delete the lexer buffer
-        yy_delete_buffer( ybs );
-
+        yyCodeBuffer = yy_scan_string( codeLiteral.c_str() );
+        // if could load
+        if( !yyCodeBuffer ) goto cleanup;
+        // set source
+        source.setCode( codeLiteral.c_str() );
     }
-    */
-
-    // check length | 1.4.1.0 (ge) added
-    t_CKUINT len = strlen( fname );
-    if( len > MAX_FILENAME_LEN )
+    else
     {
-        EM_error2( 0, "filename length (%d) exceeds max (%d) set by compiler...",
-                   len, MAX_FILENAME_LEN );
-        EM_error2( 0, "filename in question: '%s'", fname );
-        EM_error2( 0, "compiler bailing out..." );
-        return FALSE;
-    }
-    // remember filename
-    strcpy( g_filename, fname );
+        // test it
+        if( !fd )
+        {
+            // open, could potentially change g_filename
+            fd = ck_openFileAutoExt( g_filename, ".ck" );
+            // check file open result
+            if( !fd ) // if couldn't open
+            { g_filename = fname; } // revert filename
+            else if( ck_isdir(g_filename) ) // check for directory; if so, clean up
+            { EM_error2( 0, "cannot parse file: '%s' is a directory", mini(g_filename.c_str()) ); goto cleanup; }
+        }
 
-    // test it
-    if( !fd ) {
-        fd = open_cat_ck( g_filename );
-        if( !fd ) strcpy( g_filename, fname );
-        else clo = TRUE;
+        // if still none
+        if( !fd ) { EM_error2( 0, "no such file: '%s'", mini(fname.c_str()) ); goto cleanup; }
+        // set to beginning0
+        else fseek( fd, 0, SEEK_SET );
+
+        // reset yyin to fd
+        yyrestart( fd );
+        // set to initial condition | 1.5.0.5 (ge) added
+        yyinitial();
+
+        // check
+        if( yyin == NULL ) goto cleanup;
+
+        // set source
+        source.setFile( fd );
     }
 
-    // reset
-    if( EM_reset( g_filename, fd ) == FALSE ) goto cleanup;
+    // start error message for new filename
+    EM_start_filename( g_filename.c_str() );
+    // set current input source
+    EM_setCurrentFileSource( source );
 
-    // lexer/parser
-    // TODO: if( yyin ) { fclose( yyin ); yyin = NULL; }
-
-    // if no fd, open
-    if( !fd ) { fd = fopen( g_filename, "r" ); if( fd ) clo = TRUE; }
-    // if still none
-    if( !fd ) { EM_error2( 0, "no such file or directory" ); goto cleanup; }
-    // set to beginning
-    else fseek( fd, 0, SEEK_SET );
-
-    // reset yyin to fd
-    yyrestart( fd );
-
-    // check
-    if( yyin == NULL ) goto cleanup;
-
-    // TODO: clean g_program
-    g_program = NULL;
+    // ensure abstract syntax tree is clean | 1.5.0.5 (ge) added, finally
+    cleanup_AST();
 
     // parse
-    if( !(yyparse( ) == 0) ) goto cleanup;
+    if( !(yyparse() == 0) ) goto cleanup;
 
     // flag success
     ret = TRUE;
 
 cleanup:
 
-    // done
-    if( clo ) fclose( fd );
+    // defer some clean up to reset_parse(), since later passes
+    // could use current file source / file descriptor; copy
+    g_fd2close = fd;
+
+    // flush
+    // yyflush();
+
+    // set to NULL | 1.5.0.5 (ge) added to reset yyin
+    yyin = NULL;
+
+    // clean up lexer buffer, if we used one
+    if( yyCodeBuffer )
+    { yy_delete_buffer( yyCodeBuffer ); yyCodeBuffer = NULL; }
 
     return ret;
 }
@@ -273,14 +188,80 @@ cleanup:
 
 
 
+//------------------------------------------------------------------------------
+// name: fd2parse_cleanup()
+// desc: clean up fd2parse
+//------------------------------------------------------------------------------
+void fd2parse_cleanup()
+{
+    // check
+    if( g_fd2parse && g_fd2autoClose )
+    {
+        // log
+        EM_log( CK_LOG_FINEST, "fd2parse_clean() closing FILE descriptor [0x%x]...", (t_CKUINT)g_fd2parse );
+        fclose( g_fd2parse );
+    }
+
+    // reset
+    g_fd2parse = NULL;
+    g_fd2autoClose = FALSE;
+}
+
+
+
+
+//------------------------------------------------------------------------------
+// name: fd2parse_set()
+// desc: special FILE input mode | added 1.5.0.5 (ge)
+// set an already open FILE descriptor `fd` for one time use
+// by the next call to go(), which will use `fd` as the input
+// to the parser (NOTE in any invocation of go(), `codeLiteral`
+// and `fd` should not both be non-empty, otherwise a warning
+// will be output and the `codeLiteral` will take precedence
+// and the `fd` will be cleaned up and skipped
+// MEMORY: if `autoClose` is true, the compiler will automatically
+// call fclose() on `fd` on the next call to go(), regardless of
+// the aforementioned conflict with `codeLiteral`
+//------------------------------------------------------------------------------
+void fd2parse_set( FILE * fd, t_CKBOOL autoClose )
+{
+    // clean up
+    fd2parse_cleanup();
+
+    // copy
+    g_fd2parse = fd;
+    g_fd2autoClose = autoClose;
+}
+
+
+
+
 //------------------------------------------------------------------------------
 // name: reset_parse()
-// desc: ...
+// desc: reset parse after a parse
 //------------------------------------------------------------------------------
 void reset_parse( )
 {
+    // reset the current input source
+    EM_cleanupCurrentFileSource();
+
+    // clean up the file descriptor
+    if( g_fd2close )
+    {
+        // check fd2parse (could be the same)
+        if( g_fd2parse ) { fd2parse_cleanup(); }
+        else { fclose( g_fd2close ); }
+        // zero out either way
+        g_fd2close = NULL;
+    }
+
     // empty file name
-    EM_change_file( NULL );
+    EM_reset_filename();
+    // clear current ChucK pointer; no dangling references
+    EM_set_current_chuck( NULL );
+
+    // clean up the AST
+    cleanup_AST();
 }
 
 
@@ -306,6 +287,8 @@ t_CKBOOL SyntaxQuery::parseLine( const std::string & line, SyntaxTokenList & tok
 // absyn to string functions
 //-----------------------------------------------------------------------------
 #include "chuck_type.h"
+
+// expression to string conversion
 string absyn_exp2str( a_Exp exp, t_CKBOOL iterate = true );
 string absyn_binary2str( a_Exp_Binary binary );
 string absyn_unary2str( a_Exp_Unary unary );
@@ -320,6 +303,105 @@ string absyn_dot_member2str( a_Exp_Dot_Member dot_member );
 string absyn_exp_if2str( a_Exp_If exp_if );
 string absyn_decl2str( a_Exp_Decl decl );
 
+// statement to string conversion
+string absyn_stmt2str( a_Stmt stmt );
+string absyn_if2str( a_Stmt_If stmt_if );
+string absyn_for2str( a_Stmt_For stmt_for );
+string absyn_foreach2str( a_Stmt_ForEach stmt_foreach );
+string absyn_while2str( a_Stmt_While stmt_while );
+string absyn_until2str( a_Stmt_Until stmt_until );
+string absyn_loop2str( a_Stmt_Loop stmt_loop );
+string absyn_break2str( a_Stmt_Break stmt_break );
+string absyn_continue2str( a_Stmt_Continue stmt_continue );
+string absyn_return2str( a_Stmt_Return stmt_return );
+string absyn_code2str( a_Stmt_Code stmt_code );
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_stmt2str()
+// desc: convert abstract syntax stmt to string
+//-----------------------------------------------------------------------------
+std::string absyn2str( a_Stmt stmt )
+{
+    return absyn_stmt2str( stmt );
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_stmt2str()
+// desc: convert abstract syntax stmt to string
+//-----------------------------------------------------------------------------
+string absyn_stmt2str( a_Stmt stmt )
+{
+    // check
+    if( !stmt ) return "";
+
+    // str to be constructed
+    string str = "";
+
+    // loop over statements, check by type
+    switch( stmt->s_type )
+    {
+        case ae_stmt_exp:  // expression statement
+            str = absyn_exp2str( stmt->stmt_exp );
+            break;
+
+        case ae_stmt_if:  // if statement
+            str = absyn_if2str( &stmt->stmt_if );
+            break;
+
+        case ae_stmt_for:  // for statement
+            str = absyn_for2str( &stmt->stmt_for );
+            break;
+
+        case ae_stmt_foreach:  // for statement
+            str = absyn_foreach2str( &stmt->stmt_foreach );
+            break;
+
+        case ae_stmt_while:  // while statement
+            str = absyn_while2str( &stmt->stmt_while );
+            break;
+
+        case ae_stmt_until:  // until statement
+            str = absyn_until2str( &stmt->stmt_until );
+            break;
+
+        case ae_stmt_loop:  // loop statement
+            str = absyn_loop2str( &stmt->stmt_loop );
+            break;
+
+        case ae_stmt_break:  // break statement
+            str = absyn_break2str( &stmt->stmt_break );
+            break;
+
+        case ae_stmt_continue:  // continue statement
+            str = absyn_continue2str( &stmt->stmt_continue );
+            break;
+
+        case ae_stmt_return:  // return statement
+            str = absyn_return2str( &stmt->stmt_return );
+            break;
+
+        case ae_stmt_code:  // code segment
+            str = absyn_code2str( &stmt->stmt_code );
+            break;
+
+        // not implemented
+        case ae_stmt_switch: // switch statement
+        case ae_stmt_case: // case statement
+        case ae_stmt_gotolabel: // goto label statement
+        default: // bad
+            break;
+    }
+
+    // return the constructed str
+    return str;
+}
+
 
 
 
@@ -357,6 +439,10 @@ string name_demangle( const string & name )
 //-----------------------------------------------------------------------------
 string absyn_exp2str( a_Exp exp, t_CKBOOL iterate )
 {
+    // check
+    if( !exp ) return "";
+
+    // the string to be constructed
     string str = "";
 
     // loop over
@@ -526,11 +612,13 @@ string absyn_binary2str( a_Exp_Binary binary )
         case ae_op_chuck:
         case ae_op_unchuck:
         case ae_op_upchuck:
+        case ae_op_at_chuck:
+        case ae_op_eq:
+        case ae_op_neq:
         case ae_op_lt:
         case ae_op_gt:
         case ae_op_le:
         case ae_op_ge:
-        case ae_op_at_chuck:
             paren = false;
             spacing = true;
             break;
@@ -555,7 +643,20 @@ string absyn_binary2str( a_Exp_Binary binary )
 //-----------------------------------------------------------------------------
 string absyn_unary2str( a_Exp_Unary unary )
 {
-    return absyn_op2str(unary->op) + " " + absyn_exp2str(unary->exp);
+    string s;
+
+    // check
+    switch( unary->op )
+    {
+        case ae_op_new:
+            s = absyn_op2str(unary->op) + " " + unary->self->type->name();
+            break;
+        default:
+            s = absyn_op2str(unary->op) + " " + absyn_exp2str(unary->exp);
+            break;
+    }
+
+    return s;
 }
 
 
@@ -568,7 +669,7 @@ string absyn_unary2str( a_Exp_Unary unary )
 string absyn_cast2str( a_Exp_Cast cast )
 {
     Chuck_Type * to = cast->self->type;
-    return absyn_exp2str(cast->exp) + "$" + to->name;
+    return absyn_exp2str(cast->exp) + "$" + to->base_name;
 }
 
 
@@ -620,7 +721,7 @@ string absyn_primary2str( a_Exp_Primary primary )
         case ae_primary_vec: str = "@("+absyn_exp2str(primary->vec->args)+")"; break;
         case ae_primary_exp: str = absyn_exp2str(primary->exp); break;
         case ae_primary_hack: str = "<<< " + absyn_exp2str(primary->exp) + " >>>"; break;
-        case ae_primary_nil: str = "null"; break;
+        case ae_primary_nil: str = "(void)"; break;
         default: str ="[PRIMARY]"; break;
     }
 
@@ -722,7 +823,7 @@ string absyn_decl2str( a_Exp_Decl decl )
     while( list )
     {
         // type
-        str += list->var_decl->value->type->name + " " + (decl->type->ref && !list->var_decl->array ? "@ " : "") + list->var_decl->value->name;
+        str += list->var_decl->value->type->base_name + " " + (decl->type->ref ? "@ " : "") + list->var_decl->value->name;
         // array?
         if( list->var_decl->array )
         {
@@ -753,3 +854,126 @@ string absyn_decl2str( a_Exp_Decl decl )
 
     return str;
 }
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_if2str()
+// desc: convert abstract syntax if to string
+//-----------------------------------------------------------------------------
+string absyn_if2str( a_Stmt_If stmt_if )
+{
+    return "if( " + absyn_exp2str( stmt_if->cond ) + " )";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_for2str()
+// desc: convert abstract syntax for to string
+//-----------------------------------------------------------------------------
+string absyn_for2str( a_Stmt_For stmt_for )
+{
+    return "for( " + absyn_stmt2str( stmt_for->c1 ) + "; "
+                   + absyn_stmt2str( stmt_for->c2 ) + "; "
+                   + absyn_exp2str( stmt_for->c3 ) + " )";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_foreach2str()
+// desc: convert abstract syntax for-each to string
+//-----------------------------------------------------------------------------
+string absyn_foreach2str( a_Stmt_ForEach stmt_foreach )
+{
+    return "for( " + absyn_exp2str( stmt_foreach->theIter ) + " : "
+                   + absyn_exp2str( stmt_foreach->theArray ) + " )";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_while2str()
+// desc: convert abstract syntax while to string
+//-----------------------------------------------------------------------------
+string absyn_while2str( a_Stmt_While stmt_while )
+{
+    return "while( " + absyn_exp2str( stmt_while->cond ) + " )";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_until2str()
+// desc: convert abstract syntax until to string
+//-----------------------------------------------------------------------------
+string absyn_until2str( a_Stmt_Until stmt_until )
+{
+    return "until( " + absyn_exp2str( stmt_until->cond ) + " )";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_loop2str()
+// desc: convert abstract syntax loop to string
+//-----------------------------------------------------------------------------
+string absyn_loop2str( a_Stmt_Loop stmt_loop )
+{
+    return "repeat( " + absyn_exp2str( stmt_loop->cond ) + " )";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_break2str()
+// desc: convert abstract syntax break to string
+//-----------------------------------------------------------------------------
+string absyn_break2str( a_Stmt_Break stmt_break )
+{
+    return "break;";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_continue2str()
+// desc: convert abstract syntax continue to string
+//-----------------------------------------------------------------------------
+string absyn_continue2str( a_Stmt_Continue stmt_continue )
+{
+    return "continue;";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_return2str()
+// desc: convert abstract syntax return to string
+//-----------------------------------------------------------------------------
+string absyn_return2str( a_Stmt_Return stmt_return )
+{
+    return "return " + absyn_exp2str( stmt_return->val ) + ";";
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: absyn_stmt2str()
+// desc: convert abstract syntax decl to string
+//-----------------------------------------------------------------------------
+string absyn_code2str( a_Stmt_Code stmt_code )
+{
+    return "";
+}
