@@ -52,7 +52,7 @@ typedef enum {
     te_function, te_object, te_user, te_array, te_null, te_ugen, te_uana,
     te_event, te_void, te_stdout, te_stderr, te_adc, te_dac, te_bunghole,
     te_uanablob, te_io, te_fileio, te_chout, te_cherr, te_multi,
-    te_vec3, te_vec4, te_vector // ge: added 1.3.5.3
+    te_vec3, te_vec4, te_vector, te_auto
 } te_Type;
 
 
@@ -174,7 +174,7 @@ public:
         // add for commit
         else commit_map[xid] = value;
         // add reference
-        SAFE_ADD_REF(value);
+        CK_SAFE_ADD_REF(value);
     }
 
     // lookup id
@@ -228,14 +228,24 @@ public:
     // get list of top level
     void get_toplevel( std::vector<Chuck_VM_Object *> & out, t_CKBOOL includeMangled = TRUE )
     {
-        assert( scope.size() != 0 );
-        std::map<S_Symbol, Chuck_VM_Object *>::iterator iter;
+        // pass on
+        get_level( 0, out, includeMangled );
+    }
 
+    // get list of top level
+    void get_level( int level, std::vector<Chuck_VM_Object *> & out, t_CKBOOL includeMangled = TRUE )
+    {
+        assert( scope.size() > level );
         // clear the out
         out.clear();
-        // get the front of the array
-        std::map<S_Symbol, Chuck_VM_Object *> * m = scope.front();
 
+        // our iterator
+        std::map<S_Symbol, Chuck_VM_Object *>::iterator iter;
+        // get func map
+        std::map<S_Symbol, Chuck_VM_Object *> * m = NULL;
+
+        // 1.5.0.5 (ge) modified: actually use level
+        m = scope[level];
         // go through map
         for( iter = m->begin(); iter != m->end(); iter++ )
         {
@@ -245,28 +255,20 @@ public:
             // add
             out.push_back( (*iter).second );
         }
-    }
 
-    // get list of top level
-    void get_level( int level, std::vector<Chuck_VM_Object *> & out, t_CKBOOL includeMangled = TRUE )
-    {
-        assert( scope.size() >= level );
-        std::map<S_Symbol, Chuck_VM_Object *>::iterator iter;
-
-        // clear the out
-        out.clear();
-        // get the front of the array
-        std::map<S_Symbol, Chuck_VM_Object *> * m = &commit_map;
-
-        // go through map
-        for( iter = m->begin(); iter != m->end(); iter++ )
+        // if level 0 then also include commit map | 1.5.0.5 (ge) added
+        if( level == 0 )
         {
-            // check mangled name
-            if( !includeMangled && is_mangled( std::string(S_name((*iter).first))) )
-                continue;
-
-            // add
-            out.push_back( (*iter).second );
+            m = &commit_map;
+            // go through map
+            for( iter = m->begin(); iter != m->end(); iter++ )
+            {
+                // check mangled name
+                if( !includeMangled && is_mangled( std::string(S_name((*iter).first))) )
+                    continue;
+                // add
+                out.push_back( (*iter).second );
+            }
         }
     }
 
@@ -324,9 +326,9 @@ struct Chuck_Namespace : public Chuck_VM_Object
                         class_data = NULL; class_data_size = 0; }
     // destructor
     virtual ~Chuck_Namespace() {
-        /* TODO: SAFE_RELEASE( this->parent ); */
-        /* TODO: SAFE_DELETE( pre_ctor ); */
-        /* TODO: SAFE_DELETE( dtor ); */
+        /* TODO: CK_SAFE_RELEASE( this->parent ); */
+        /* TODO: CK_SAFE_DELETE( pre_ctor ); */
+        /* TODO: CK_SAFE_DELETE( dtor ); */
     }
 
     // look up type
@@ -372,15 +374,16 @@ struct Chuck_Context : public Chuck_VM_Object
     std::string filename;
     // full filepath (if available) -- added 1.3.0.0
     std::string full_path;
-    // parse tree
-    a_Program parse_tree;
     // context namespace
     Chuck_Namespace * nspc;
-    // public class def if any
-    a_Class_Def public_class_def;
     // error - means to free nspc too
     t_CKBOOL has_error;
 
+    // AST parse tree (does not persist past context unloading)
+    a_Program parse_tree;
+    // AST public class def if any (does not persist past context unloading)
+    a_Class_Def public_class_def;
+
     // progress
     enum { P_NONE = 0, P_CLASSES_ONLY, P_ALL_DONE };
     // progress in scan / type check / emit
@@ -392,21 +395,20 @@ struct Chuck_Context : public Chuck_VM_Object
     std::vector<Chuck_VM_Object *> new_funcs;
     std::vector<Chuck_VM_Object *> new_nspc;
 
-    // commit map
-    std::map<Chuck_Namespace *, Chuck_Namespace *> commit_map;
-    // add for commit/rollback
-    void add_commit_candidate( Chuck_Namespace * nspc );
-    // commit
-    void commit();
-    // rollback
-    void rollback();
+public:
+    // decouple from AST (called when a context is compiled)
+    // called when a context is finished and being unloaded | 1.5.0.5 (ge)
+    void decouple_ast();
 
+public:
     // constructor
     Chuck_Context() { parse_tree = NULL; nspc = new Chuck_Namespace;
                       public_class_def = NULL; has_error = FALSE;
                       progress = P_NONE; }
     // destructor
     virtual ~Chuck_Context();
+
+public:
     // get the top-level code
     Chuck_VM_Code * code() { return nspc->pre_ctor; }
 
@@ -485,6 +487,8 @@ public:
     Chuck_Func * func;
     // how far nested in a class definition
     t_CKUINT class_scope;
+    // are we in a spork operation
+    t_CKBOOL sporking;
 
     // current contexts in memory
     std::vector<Chuck_Context *> contexts;
@@ -507,33 +511,35 @@ public:
 
 public:
     // REFACTOR-2017: public types
-    Chuck_Type * t_void;
-    Chuck_Type * t_int;
-    Chuck_Type * t_float;
-    Chuck_Type * t_time;
-    Chuck_Type * t_dur;
-    Chuck_Type * t_complex;
-    Chuck_Type * t_polar;
-    Chuck_Type * t_vec3;
-    Chuck_Type * t_vec4;
-    Chuck_Type * t_null;
-    Chuck_Type * t_function;
-    Chuck_Type * t_object;
-    Chuck_Type * t_array;
-    Chuck_Type * t_string;
-    Chuck_Type * t_event;
-    Chuck_Type * t_ugen;
-    Chuck_Type * t_uana;
-    Chuck_Type * t_uanablob;
-    Chuck_Type * t_shred;
-    Chuck_Type * t_io;
-    Chuck_Type * t_fileio;
-    Chuck_Type * t_chout;
-    Chuck_Type * t_cherr;
-    // Chuck_Type * t_thread;
-    Chuck_Type * t_class;
-    Chuck_Type * t_dac;
-    Chuck_Type * t_adc;
+    Chuck_Type * ckt_void;
+    Chuck_Type * ckt_auto; // 1.5.0.8
+    Chuck_Type * ckt_int;
+    Chuck_Type * ckt_float;
+    Chuck_Type * ckt_time;
+    Chuck_Type * ckt_dur;
+    Chuck_Type * ckt_complex;
+    Chuck_Type * ckt_polar;
+    Chuck_Type * ckt_vec3;
+    Chuck_Type * ckt_vec4;
+    Chuck_Type * ckt_null;
+    Chuck_Type * ckt_function;
+    Chuck_Type * ckt_object;
+    Chuck_Type * ckt_array;
+    Chuck_Type * ckt_string;
+    Chuck_Type * ckt_event;
+    Chuck_Type * ckt_ugen;
+    Chuck_Type * ckt_uana;
+    Chuck_Type * ckt_uanablob;
+    Chuck_Type * ckt_shred;
+    Chuck_Type * ckt_io;
+    Chuck_Type * ckt_fileio;
+    Chuck_Type * ckt_chout;
+    Chuck_Type * ckt_cherr;
+    Chuck_Type * ckt_class;
+    Chuck_Type * ckt_dac;
+    Chuck_Type * ckt_adc;
+
+    // Chuck_Type * ckt_thread;
 };
 
 
@@ -572,6 +578,85 @@ struct Chuck_UGen_Info : public Chuck_VM_Object
 
 
 
+//-----------------------------------------------------------------------------
+// name: struct Chuck_Value_Dependency
+// desc: a value dependency records when accessing a value; this is used to
+//       for dependency tracking of file-top-level and class-top-level variables
+//       accessed from within functions or class pre-constructor code;
+//
+// example:
+// ```
+// 5 => int foo;
+// fun void bar()
+// {
+//     // below is a dependency to foo
+//     <<< foo >>>;
+// }
+// ```
+//-----------------------------------------------------------------------------
+struct Chuck_Value_Dependency
+{
+    // value we are tracking
+    Chuck_Value * value;
+    // code position of dependency
+    t_CKUINT where;
+    // position where the use occurs (from within func or class)
+    t_CKUINT use_where;
+
+public:
+    // default
+    Chuck_Value_Dependency() : value(NULL), where(0), use_where(0) { }
+    // constructor
+    Chuck_Value_Dependency( Chuck_Value * argValue, t_CKUINT argUseWhere = 0 );
+    // copy constructor | NOTE: needed to properly ref-count
+    Chuck_Value_Dependency( const Chuck_Value_Dependency & rhs );
+    // destructor
+    virtual ~Chuck_Value_Dependency();
+};
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: struct Chuck_Value_Dependency_Graph
+// desc: data structure of value dependencies, direct and remote
+//-----------------------------------------------------------------------------
+struct Chuck_Value_Dependency_Graph
+{
+public:
+    // add a direct dependency
+    void add( const Chuck_Value_Dependency & dep );
+    // add a remote (recursive) dependency
+    void add( Chuck_Value_Dependency_Graph * graph );
+    // look for a dependency that occurs AFTER a particular code position
+    // this function crawls the graph, taking care in the event of cycles
+    const Chuck_Value_Dependency * locate( t_CKUINT pos, t_CKBOOL isClassDef = FALSE );
+
+public:
+    // constructor
+    Chuck_Value_Dependency_Graph() : token(0) { }
+
+protected:
+    // search token, for cycle detection (not reentrant, thread-wise)
+    t_CKUINT token;
+    // vector of dependencies
+    std::vector<Chuck_Value_Dependency> directs;
+    // vector of recursive dependency graphs
+    // take care regarding circular dependency
+    std::vector<Chuck_Value_Dependency_Graph *> remotes;
+
+protected:
+    // locate non-recursive
+    const Chuck_Value_Dependency * locateLocal( t_CKUINT pos, t_CKBOOL isClassDef );
+    // reset search tokens
+    void resetRecursive( t_CKUINT value = 0 );
+    // locate recursive
+    const Chuck_Value_Dependency * locateRecursive( t_CKUINT pos, t_CKBOOL isClassDef, t_CKUINT searchToken = 1 );
+};
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: struct Chuck_Type
 // desc: class containing information about a type
@@ -581,8 +666,8 @@ struct Chuck_Type : public Chuck_Object
 {
     // type id
     te_Type xid;
-    // type name
-    std::string name;
+    // type name (FYI use this->str() for full name including []s for array)
+    std::string base_name;
     // type parent (could be NULL)
     Chuck_Type * parent;
     // size (in bytes)
@@ -599,8 +684,6 @@ struct Chuck_Type : public Chuck_Object
     Chuck_Namespace * info;
     // func info
     Chuck_Func * func;
-    // def
-    a_Class_Def def;
     // ugen
     Chuck_UGen_Info * ugen_info;
     // copy
@@ -616,14 +699,17 @@ struct Chuck_Type : public Chuck_Object
     // origin hint
     te_Origin originHint;
 
+    // reference to environment RE-FACTOR 2017
+    Chuck_Env * env_ref;
+
+    // dependency tracking | 1.5.0.8
+    Chuck_Value_Dependency_Graph depends;
+
     // documentation
     std::string doc;
     // example files
     std::vector<std::string> examples;
 
-    // reference to environment RE-FACTOR 2017
-    Chuck_Env * m_env;
-
 public:
     // constructor
     Chuck_Type( Chuck_Env * env,
@@ -641,8 +727,8 @@ public:
     Chuck_Type * copy( Chuck_Env * env ) const;
 
 public:
-    // to string
-    const std::string & str();
+    // to string: the full name of this type, e.g., "UGen" or "int[][]"
+    const std::string & name();
     // to c string
     const char * c_name();
 
@@ -658,9 +744,9 @@ public: // apropos | 1.4.1.0 (ge)
 
 public: // dump | 1.4.1.1 (ge)
     // generate object state; output to console
-    void dump( Chuck_Object * obj );
+    void dump_obj( Chuck_Object * obj );
     // generate object state; output to string
-    void dump( Chuck_Object * obj, std::string & output );
+    void dump_obj( Chuck_Object * obj, std::string & output );
 
 protected: // apropos-related helper function
     // dump top level info
@@ -713,33 +799,21 @@ struct Chuck_Value : public Chuck_VM_Object
     // overloads
     t_CKINT func_num_overloads;
 
+    // dependency tracking | 1.5.0.8 (ge) added
+    // code position of where this value is considered initialized
+    // NOTE sed to determine dependencies within a file context
+    t_CKUINT depend_init_where; // 1.5.0.8
+
     // documentation
     std::string doc;
 
+public:
     // constructor
     Chuck_Value( Chuck_Type * t, const std::string & n, void * a = NULL,
                  t_CKBOOL c = FALSE, t_CKBOOL acc = 0, Chuck_Namespace * o = NULL,
-                 Chuck_Type * oc = NULL, t_CKUINT s = 0 )
-    { type = t; SAFE_ADD_REF(type); // add reference
-      name = n; offset = s;
-      is_const = c; access = acc;
-      owner = o; SAFE_ADD_REF(o); // add reference
-      owner_class = oc; SAFE_ADD_REF(oc); // add reference
-      addr = a; is_member = FALSE;
-      is_static = FALSE; is_context_global = FALSE;
-      is_decl_checked = TRUE; // only set to false in certain cases
-      is_global = FALSE;
-      func_ref = NULL; func_num_overloads = 0; }
-
+                 Chuck_Type * oc = NULL, t_CKUINT s = 0 );
     // destructor
-    virtual ~Chuck_Value()
-    {
-        // release
-        // SAFE_RELEASE( type );
-        // SAFE_RELEASE( owner ):
-        // SAFE_RELEASE( owner_class );
-        // SAFE_RELEASE( func_ref );
-    }
+    virtual ~Chuck_Value();
 };
 
 
@@ -755,8 +829,8 @@ struct Chuck_Func : public Chuck_VM_Object
     std::string name;
     // base name (without the designation, e.g., "dump"); 1.4.1.0
     std::string base_name;
-    // func def from parser
-    a_Func_Def def;
+    // human readable function signature: e.g., void Object.func( int foo, float bar[] );
+    std::string signature() const;
     // code (included imported)
     Chuck_VM_Code * code;
     // imported code
@@ -767,24 +841,55 @@ struct Chuck_Func : public Chuck_VM_Object
     t_CKBOOL is_static;
     // virtual table index
     t_CKUINT vt_index;
-    // rember value
+    // remember value
     Chuck_Value * value_ref;
     // for overloading
     Chuck_Func * next;
     // for overriding
     Chuck_Value * up;
 
+    // dependency tracking | 1.5.0.8
+    Chuck_Value_Dependency_Graph depends;
+
     // documentation
     std::string doc;
 
+protected:
+    // AST func def from parser | 1.5.0.5 (ge) moved to protected
+    // access through funcdef_*() functions
+    a_Func_Def m_def;
+
+public:
+    // connect (called when this func is type checked)
+    void funcdef_connect( a_Func_Def f );
+    // severe references to AST (called after compilation, before AST cleanup)
+    // make a fresh partial deep copy of m_def ONLY IF it is owned by AST
+    void funcdef_decouple_ast();
+    // cleanup funcdef (if/when this function is cleaned up)
+    void funcdef_cleanup();
+
+public:
+    // get the func def (do not keep a reference to this...
+    // as contents may shift during and after compilation)
+    a_Func_Def def() const { return m_def; }
+
+public:
     // constructor
-    Chuck_Func() { def = NULL; code = NULL; is_member = FALSE; is_static = FALSE,
-        vt_index = CK_NO_VALUE; value_ref = NULL; /*dl_code = NULL;*/ next = NULL;
-        up = NULL; }
+    Chuck_Func()
+    {
+        m_def = NULL;
+        code = NULL;
+        is_member = FALSE;
+        is_static = FALSE,
+        vt_index = CK_NO_VALUE;
+        value_ref = NULL;
+        /*dl_code = NULL;*/
+        next = NULL;
+        up = NULL;
+    }
 
     // destructor
-    virtual ~Chuck_Func()
-    { }
+    virtual ~Chuck_Func();
 };
 
 
@@ -905,6 +1010,8 @@ Chuck_Value * type_engine_find_value( Chuck_Env * env, const std::string & xid,
 Chuck_Namespace * type_engine_find_nspc( Chuck_Env * env, a_Id_List path );
 // convert a vector of type names to a vector of Types | 1.5.0.0 (ge) added
 void type_engine_names2types( Chuck_Env * env, const std::vector<std::string> & typeNames, std::vector<Chuck_Type *> & types );
+// check and process auto types
+t_CKBOOL type_engine_infer_auto( Chuck_Env * env, a_Exp_Decl decl, Chuck_Type * type );
 
 
 //-----------------------------------------------------------------------------
@@ -946,37 +1053,4 @@ std::string arglist2string( a_Arg_List list );
 
 
 
-//-----------------------------------------------------------------------------
-// REFACTOR-2017: exile! these default types now stored in env
-//-----------------------------------------------------------------------------
-//extern Chuck_Type t_void;
-//extern Chuck_Type t_int;
-//extern Chuck_Type t_float;
-//extern Chuck_Type t_time;
-//extern Chuck_Type t_dur;
-//extern Chuck_Type t_complex;
-//extern Chuck_Type t_polar;
-//extern Chuck_Type t_vec3; // ge: added 1.3.5.3
-//extern Chuck_Type t_vec4; // ge: added 1.3.5.3
-//extern Chuck_Type t_vector;
-//extern Chuck_Type t_object;
-//extern Chuck_Type t_null;
-//extern Chuck_Type t_string;
-//extern Chuck_Type t_array;
-//extern Chuck_Type t_shred;
-//extern Chuck_Type t_thread;
-//extern Chuck_Type t_function;
-//extern Chuck_Type t_class;
-//extern Chuck_Type t_event;
-//extern Chuck_Type t_io;
-//extern Chuck_Type t_fileio;
-//extern Chuck_Type t_chout;
-//extern Chuck_Type t_cherr;
-//extern Chuck_Type t_ugen;
-//extern Chuck_Type t_uana;
-//extern Chuck_Type t_uanablob;
-
-
-
-
 #endif
