@@ -41,21 +41,26 @@ extern "C" {
 #endif
 
 // 1.4.1.1 (ge) removed -- seems no longer needed on more modern windows
-#ifdef __PLATFORM_WIN32__
+#ifdef __PLATFORM_WINDOWS__
 #ifdef __CK_MATH_DEFINE_ROUND_TRUNC__
     double round( double a );
     double trunc( double a );
 #endif
 #endif
 
-// 1.4.1.0 (ge) need this since __WINDOWS_MODERN__ can be defined but remainder still not found (VC++ 2010)
+// 1.4.1.0 (ge) need this since remainder still not found (VC++ 2010)
 double ck_remainder( double a, double b );
 
 
-// chuck random wrapper
+// chuck random wrapper, return t_CKINT in [0,CK_RANDOM_MAX]
 t_CKINT ck_random();
+// chuck random float wrapper, return t_CKFLOAT in [0,1]
+t_CKFLOAT ck_random_f();
 // chuck srandom wrapper
 void ck_srandom( unsigned seed );
+// randomizer wrapper (use this instead of seeding for initial)
+void ck_randomize();
+
 
 //-----------------------------------------------------------------------------
 #ifndef __OLDSCHOOL_RANDOM__
@@ -64,32 +69,32 @@ void ck_srandom( unsigned seed );
 //-----------------------------------------------------------------------------
 #else // __OLDSCHOOL_RANDOM__ (enable only for compatibility pre-c++11)
 //-----------------------------------------------------------------------------
-#ifdef __PLATFORM_WIN32__
-  #define CK_RANDOM_MAX RAND_MAX
-#else
-  #define CK_RANDOM_MAX 0x7fffffff
-#endif
+  #ifndef __PLATFORM_WINDOWS__
+    #define CK_RANDOM_MAX 0x7fffffff
+  #else // __PLATFORM_WINDOWS__
+    #define CK_RANDOM_MAX RAND_MAX
+  #endif
 //-----------------------------------------------------------------------------
 #endif // __OLDSCHOOL_RANDOM__
 //-----------------------------------------------------------------------------
 
 
 // mtof
-double mtof( double f );
+double ck_mtof( double f );
 // ftom
-double ftom( double f );
+double ck_ftom( double f );
 // powtodb
-double powtodb( double f );
+double ck_powtodb( double f );
 // rmstodb
-double rmstodb( double f );
+double ck_rmstodb( double f );
 // dbtopow
-double dbtopow( double f );
+double ck_dbtopow( double f );
 // dbtorms
-double dbtorms( double f );
+double ck_dbtorms( double f );
 // nextpow2
-unsigned long nextpow2( unsigned long i );
+unsigned long ck_nextpow2( unsigned long i );
 // ensurepow2
-unsigned long ensurepow2( unsigned long i );
+unsigned long ck_ensurepow2( unsigned long i );
 
 #if defined (__cplusplus) || defined(_cplusplus)
 }
