@@ -34,10 +34,11 @@
 #include "chuck.h"
 #include "chuck_audio.h"
 #include "chuck_console.h"
+#include "util_platforms.h"
 #include "util_string.h"
 #include <signal.h>
 
-#if defined(__PLATFORM_WIN32__)
+#if defined(__PLATFORM_WINDOWS__)
   #include <windows.h>
 #else
   #include <unistd.h>
@@ -54,14 +55,14 @@ using namespace std;
 //-----------------------------------------------------------------------------
 t_CKBOOL init_shell( Chuck_Shell * shell, Chuck_Shell_UI * ui, Chuck_VM * vm );
 void * shell_cb( void * p );
-bool go( int argc, const char ** argv );
-void global_cleanup();
+t_CKBOOL go( int argc, const char ** argv );
+t_CKBOOL global_cleanup();
 void all_stop();
 void all_detach();
 void usage();
 void uh();
 t_CKBOOL get_count( const char * arg, t_CKUINT * out );
-void cb( t_CKSAMPLE * in, t_CKSAMPLE * out, t_CKUINT numFrames,
+void cb( SAMPLE * in, SAMPLE * out, t_CKUINT numFrames,
          t_CKUINT numInChans, t_CKUINT numOutChans, void * data );
 
 // C functions
@@ -104,10 +105,10 @@ t_CKUINT g_num_vms_running = 0;
 
 
 // priority stuff
-#if defined(__MACOSX_CORE__)
+#if defined(__PLATFORM_APPLE__)
 t_CKINT g_priority = XThreadUtil::get_max_priority(); // was 80
 t_CKINT g_priority_low = XThreadUtil::get_min_priority(); // was 60
-#elif defined(__PLATFORM_WIN32__) && !defined(__WINDOWS_PTHREAD__)
+#elif defined(__PLATFORM_WINDOWS__) && !defined(__WINDOWS_PTHREAD__)
 t_CKINT g_priority = THREAD_PRIORITY_HIGHEST;
 t_CKINT g_priority_low = THREAD_PRIORITY_HIGHEST;
 #else
@@ -120,7 +121,7 @@ t_CKINT g_priority_low = 0x7fffffff;
 
 //-----------------------------------------------------------------------------
 // name: main()
-// desc: ...
+// desc: command line chuck entry point
 //-----------------------------------------------------------------------------
 #ifndef __CHUCK_NO_MAIN__
 int main( int argc, const char ** argv )
@@ -132,7 +133,13 @@ int chuck_main( int argc, const char ** argv )
     go( argc, argv );
 
     // clean up
-    global_cleanup();
+    if( !global_cleanup() )
+    {
+        // we got here possibly because global_cleanup() is underway on
+        // another thread; wait a bit and give actual cleaner a chance to
+        // finish | 1.5.0.4 (ge) added
+        ck_usleep( 50000 );
+    }
     
     return 0;
 }
@@ -144,7 +151,7 @@ int chuck_main( int argc, const char ** argv )
 // name: cb()
 // desc: audio callback
 //-----------------------------------------------------------------------------
-void cb( t_CKSAMPLE * in, t_CKSAMPLE * out, t_CKUINT numFrames,
+void cb( SAMPLE * in, SAMPLE * out, t_CKUINT numFrames,
          t_CKUINT numInChans, t_CKUINT numOutChans, void * data )
 {
     // TODO: check channel numbers
@@ -163,45 +170,70 @@ void cb( t_CKSAMPLE * in, t_CKSAMPLE * out, t_CKUINT numFrames,
 static void version()
 {
     CK_FPRINTF_STDERR( "\n" );
+    CK_FPRINTF_STDERR( "%s", TC::reset().c_str() );
     CK_FPRINTF_STDERR( "chuck version: %s\n", ChucK::version() );
     
     // platform string
     string platform = "";
+    // drivers string
+    string drivers = "";
     // binary architecture string;
     string archs = "";
+    // print space
+    t_CKINT space = 0;
 
-#if defined(__PLATFORM_WIN32__)
-    #if defined(__WINDOWS_ASIO__) && defined(__WINDOWS_DS__)
-        platform = "microsoft windows + DS + ASIO";
-    #elif defined(__WINDOWS_ASIO__)
-        platform = "microsoft windows + ASIO";
-    #elif defined(__WINDOWS_DS__)
-        platform = "microsoft windows + DS";
-    #else
-        platform = "microsoft windows + (?)";
-    #endif
-#elif defined(__LINUX_ALSA__)
-    platform = "linux (alsa)";
-#elif defined(__LINUX_OSS__)
-    platform = "linux (oss)";
-#elif defined(__LINUX_JACK__) || defined(__UNIX_JACK__)
-    platform = "linux (jack)";
-#elif defined(__LINUX_PULSE__)
-    platform = "linux (pulse)";
-#elif defined(__MACOSX_CORE__) && defined(__LITTLE_ENDIAN__)
+#if defined(__PLATFORM_WINDOWS__)
+    platform = "Microsoft Windows";
+#elif defined(__PLATFORM_LINUX__)
+    platform = "Linux";
+#elif defined(__PLATFORM_APPLE__)
     platform = "macOS";
-#elif defined(__MACOSX_CORE__)
-    platform = "macOS (powerpc)";
-#else
-    platform = "unspecified platform";
+#elif defined(__PLATFORM_EMSCRIPTEN__)
+    platform = "web";
 #endif
 
+// windows related
+#if defined(__WINDOWS_ASIO__)
+    drivers += string( space ? " | " : "" ) + "ASIO"; space++;
+#endif
+#if defined(__WINDOWS_DS__)
+    drivers += string(space ? " | " : "") + "DirectSound"; space++;
+#endif
+#if defined(__WINDOWS_WASAPI__)
+    drivers += string(space ? " | " : "") + "WASAPI"; space++;
+#endif
+
+// mac
+#if defined(__MACOSX_CORE__)
+    drivers += string( space ? " | " : "" ) + "CoreAudio"; space++;
+#endif
+
+// linux related
+#if defined(__LINUX_ALSA__)
+    drivers += string(space ? " | " : "") + "ALSA"; space++;
+#endif
+#if defined(__LINUX_OSS__)
+    drivers += string(space ? " | " : "") + "OSS"; space++;
+#endif
+#if defined(__LINUX_PULSE__)
+    drivers += string(space ? " | " : "") + "PulseAudio"; space++;
+#endif
+
+// linux / neutral
+#if defined(__LINUX_JACK__) || defined(__UNIX_JACK__)
+    drivers += string( space ? " | " : "" ) + "JACK"; space++;
+#endif
+
+    // no drivers
+    if( space == 0 ) drivers = "(none)";
+
 // check for universal binary
-#if defined(__MACOSX_UB__)
+#if defined(__MACOS_UB__)
     archs = " [universal binary]";
 #endif
 
-    CK_FPRINTF_STDERR( "   %s : %ld-bit%s\n", platform.c_str(), machine_intsize(), archs.c_str() );
+    CK_FPRINTF_STDERR( "   %s | %ld-bit%s\n", platform.c_str(), machine_intsize(), archs.c_str() );
+    CK_FPRINTF_STDERR( "   audio driver%s: %s\n", space>1 ? "s" : "", drivers.c_str() );
     CK_FPRINTF_STDERR( "   http://chuck.cs.princeton.edu/\n" );
     CK_FPRINTF_STDERR( "   http://chuck.stanford.edu/\n\n" );
 }
@@ -215,15 +247,20 @@ static void version()
 void usage()
 {
     // (note: optional colon added 1.3.0.0)
-    CK_FPRINTF_STDERR( "usage: chuck --[options|commands] [+-=^] file1 file2 file3 ...\n" );
-    CK_FPRINTF_STDERR( "   [options] = halt|loop|audio|silent|dump|nodump|about|probe|\n" );
-    CK_FPRINTF_STDERR( "               channels:<N>|out:<N>|in:<N>|dac:<N>|adc:<N>|driver:<name>|\n" );
-    CK_FPRINTF_STDERR( "               srate:<N>|bufsize:<N>|bufnum:<N>|shell|empty|\n" );
-    CK_FPRINTF_STDERR( "               remote:<hostname>|port:<N>|verbose:<N>|level:<N>|\n" );
-    CK_FPRINTF_STDERR( "               callback|deprecate:{stop|warn|ignore}|\n" );
-    CK_FPRINTF_STDERR( "               chugin-load:{on|off}|chugin-path:<path>|chugin:<name>\n" );
-    CK_FPRINTF_STDERR( "   [commands] = add|remove|replace|remove.all|status|time|kill\n" );
-    CK_FPRINTF_STDERR( "   [+-=^] = shortcuts for add, remove, replace, status\n" );
+    CK_FPRINTF_STDERR( "%s", TC::reset().c_str() );
+    CK_FPRINTF_STDERR( "\nusage: %s --[%s|%s] [%s] file1 file2 ...\n\n", "chuck", TC::orange("options").c_str(), TC::blue("commands").c_str(), TC::blue("+-=^").c_str() );
+    CK_FPRINTF_STDERR( "%s", TC::set_orange().c_str() );
+    CK_FPRINTF_STDERR( "    [options] = halt|loop|audio|silent|dump|nodump|about|probe|\n" );
+    CK_FPRINTF_STDERR( "                channels:<N>|out:<N>|in:<N>|dac:<N>|adc:<N>|driver:<name>|\n" );
+    CK_FPRINTF_STDERR( "                srate:<N>|bufsize:<N>|bufnum:<N>|shell|empty|\n" );
+    CK_FPRINTF_STDERR( "                remote:<hostname>|port:<N>|verbose:<N>|level:<N>|\n" );
+    CK_FPRINTF_STDERR( "                callback|deprecate:{stop|warn|ignore}|chugin-probe|\n" );
+    CK_FPRINTF_STDERR( "                chugin-load:{on|off}|chugin-path:<path>|chugin:<name>\n" );
+    CK_FPRINTF_STDERR( "                --color:{on|off}|--no-color\n" );
+    CK_FPRINTF_STDERR( "%s", TC::set_blue().c_str() );
+    CK_FPRINTF_STDERR( "   [commands] = add|remove|replace|remove.all|status|time|\n" );
+    CK_FPRINTF_STDERR( "                clear.vm|reset.id|abort.shred|exit\n" );
+    CK_FPRINTF_STDERR( "       [+-=^] = shortcuts for add, remove, replace, status\n" );
     version();
 }
 
@@ -257,15 +294,24 @@ t_CKBOOL get_count( const char * arg, t_CKUINT * out )
 
 //-----------------------------------------------------------------------------
 // name: signal_int()
-// desc: handler for ctrl-c (SIGINT).  NB: on windows this is triggered
+// desc: handler for ctrl-c (SIGINT). NB: on windows this is triggered
 //       in a separate thread. global_cleanup requires a mutex.
+//       ... or another synchronization mechanism
 //-----------------------------------------------------------------------------
 extern "C" void signal_int( int sig_num )
 {
     // log
     CK_FPRINTF_STDERR( "[chuck]: cleaning up...\n" );
-    // clean up everything!
-    global_cleanup();
+
+    // clean up
+    if( !global_cleanup() )
+    {
+        // we got here possibly because global_cleanup() is underway on
+        // another thread; wait a bit and give actual cleaner a chance to
+        // finish | 1.5.0.4 (ge) added
+        ck_usleep( 50000 );
+    }
+
     // exit with code
     exit( 130 ); // 130 -> terminated by ctrl-c
 }
@@ -279,9 +325,11 @@ extern "C" void signal_int( int sig_num )
 //-----------------------------------------------------------------------------
 extern "C" void signal_pipe( int sig_num )
 {
-    fprintf( stderr, "[chuck]: sigpipe handled - broken pipe (no connection)...\n" );
+    CK_FPRINTF_STDERR( "[chuck]: sigpipe handled - broken pipe (no connection)...\n" );
     if( g_sigpipe_mode )
     {
+        // log
+        CK_FPRINTF_STDERR( "[chuck]: cleaning up...\n" );
         // clean up everything!
         global_cleanup();
         // later
@@ -296,7 +344,7 @@ extern "C" void signal_pipe( int sig_num )
 // name: global_cleanup()
 // desc: ...
 //-----------------------------------------------------------------------------
-void global_cleanup()
+t_CKBOOL global_cleanup()
 {
     // make sure we don't double clean
     static bool s_already_cleaning = false;
@@ -307,7 +355,7 @@ void global_cleanup()
         // log | 1.5.0.0 (ge) added
         EM_log( CK_LOG_INFO, "additional global cleanup not needed..." );
         // done
-        return;
+        return FALSE;
     }
 
     // log | 1.5.0.0 (ge) added
@@ -327,20 +375,20 @@ void global_cleanup()
     ChucK::globalCleanup();
     
     // delete the chuck
-    SAFE_DELETE( the_chuck );
+    CK_SAFE_DELETE( the_chuck );
 
     // delete shell and set to NULL | 1.4.1.0
-    if( g_enable_shell ) SAFE_DELETE( g_shell );
+    if( g_enable_shell ) CK_SAFE_DELETE( g_shell );
 
     // wait for the shell, if it is running
     // does the VM reset its priority to normal before exiting?
     //if( g_enable_shell )
     //    while( g_shell != NULL )
-    //        usleep(10000);
+    //        ck_usleep(10000);
 
     // REFACTOR-2017 TODO: Cancel otf, le_cb threads? Does this happen in ~ChucK()?
     // things don't work so good on windows...
-#if !defined(__PLATFORM_WIN32__) || defined(__WINDOWS_PTHREAD__)
+#if !defined(__PLATFORM_WINDOWS__) || defined(__WINDOWS_PTHREAD__)
 //    if( g_tid_otf ) pthread_cancel( g_tid_otf );
 //    if( g_tid_whatever ) pthread_cancel( g_tid_whatever );
 #else
@@ -350,8 +398,9 @@ void global_cleanup()
 
     // 1.5.0.0 (ge) | commented
     // s_mutex.release();
-}
 
+    return TRUE;
+}
 
 
 
@@ -455,13 +504,13 @@ void * shell_cb( void * p )
 // name: go()
 // desc: parse args, init stuff, run!
 //-----------------------------------------------------------------------------
-bool go( int argc, const char ** argv )
+t_CKBOOL go( int argc, const char ** argv )
 {
     t_CKBOOL vm_halt = TRUE;
-    t_CKINT srate = SAMPLE_RATE_DEFAULT;
+    t_CKINT srate = CK_SAMPLE_RATE_DEFAULT;
     t_CKBOOL force_srate = FALSE; // added 1.3.1.2
-    t_CKINT buffer_size = BUFFER_SIZE_DEFAULT;
-    t_CKINT num_buffers = NUM_BUFFERS_DEFAULT;
+    t_CKINT buffer_size = CK_BUFFER_SIZE_DEFAULT;
+    t_CKINT num_buffers = CK_NUM_BUFFERS_DEFAULT;
     t_CKINT dac = 0;
     t_CKINT adc = 0;
     std::string dac_name = ""; // added 1.3.0.0
@@ -471,6 +520,9 @@ bool go( int argc, const char ** argv )
     t_CKINT adc_chans_before_rtaudio = adc_chans; // added 1.5.0.0
     t_CKBOOL dump = FALSE;
     t_CKBOOL probe = FALSE;
+    t_CKBOOL probe_chugs = FALSE;
+    t_CKBOOL doAbout = FALSE;
+    t_CKBOOL doVersion = FALSE;
     t_CKBOOL set_priority = FALSE;
     t_CKBOOL auto_depend = FALSE;
     t_CKBOOL block = FALSE;
@@ -486,15 +538,29 @@ bool go( int argc, const char ** argv )
     t_CKINT  chugin_load = 1; // 1 == auto (variable added 1.3.0.0)
     // whether to make this new VM the one that receives OTF commands
     t_CKBOOL update_otf_vm = TRUE;
+    // whether to print code quotes for compiler message | 1.5.0.5 (ge) added
+    t_CKBOOL suppress_error_quote = FALSE;
+    // color terminal output | 1.5.0.5 (ge) added
+    t_CKBOOL colorTerminal = TRUE;
     string   filename = "";
     vector<string> args;
-    const char * audioDriver = NULL;
-    // default driver name | 1.5.0.1 (ge) added
-    if( g_enable_realtime_audio )
-        audioDriver = ChuckAudio::defaultDriverName();
-    // dac and adc devices names | 1.5.0.0 (ge) added
+    // audio driver | 1.5.0.0
+    string   audio_driver = "";
+    // dac and adc devices names | 1.5.0.0
     string   dac_device_name = "";
     string   adc_device_name = "";
+    // default driver name | 1.5.0.1
+    if( g_enable_realtime_audio )
+        audio_driver = ChuckAudio::defaultDriverName();
+    // error message string
+    string errorMessage1;
+    string errorMessage2;
+
+    // do any ANSI code config | 1.5.0.5 (ge) added
+    // (e.g., windows need to explicitly be configured to process
+    // escape sequences, e.g., for color console output)
+    // NOTE: do this early in case printing is desired
+    ck_configANSI_ESCcodes();
 
     // list of search pathes (added 1.3.0.0)
     std::list<std::string> dl_search_path;
@@ -515,12 +581,12 @@ bool go( int argc, const char ** argv )
     parse_path_list( initial_chugin_path, dl_search_path );
     // list of individually named chug-ins (added 1.3.0.0)
     std::list<std::string> named_dls;
-    
+
 #if defined(__DISABLE_WATCHDOG__)
     do_watchdog = FALSE;
-#elif defined(__MACOSX_CORE__)
+#elif defined(__PLATFORM_APPLE__)
     do_watchdog = TRUE;
-#elif defined(__PLATFORM_WIN32__) && !defined(__WINDOWS_PTHREAD__)
+#elif defined(__PLATFORM_WINDOWS__) && !defined(__WINDOWS_PTHREAD__)
     do_watchdog = TRUE;
 #else
     do_watchdog = FALSE;
@@ -589,7 +655,7 @@ bool go( int argc, const char ** argv )
             else if( !strncmp(argv[i], "-n", 2) )
                 num_buffers = atoi( argv[i]+2 ) > 0 ? atoi( argv[i]+2 ) : num_buffers;
             else if( !strncmp(argv[i], "--driver:", 9))
-                audioDriver = argv[i]+9;
+                audio_driver = argv[i]+9;
             else if( !strncmp(argv[i], "--dac:", 6) ) // (added 1.3.0.0)
             {
                 // advance pointer to beginning of argument
@@ -612,10 +678,9 @@ bool go( int argc, const char ** argv )
                 }
                 else
                 {
-                    // error
-                    CK_FPRINTF_STDERR( "[chuck]: invalid arguments for '--dac:'\n" );
-                    CK_FPRINTF_STDERR( "[chuck]: (see 'chuck --help' for more info)\n" );
-                    exit( 1 );
+                    errorMessage1 = "invalid arguments for '--dac:'";
+                    errorMessage2 = "(see 'chuck --help' for more info)";
+                    break;
                 }
             }
             else if( !strncmp(argv[i], "--dac", 5) )
@@ -643,9 +708,9 @@ bool go( int argc, const char ** argv )
                 else
                 {
                     // error
-                    CK_FPRINTF_STDERR( "[chuck]: invalid arguments for '--adc:'\n" );
-                    CK_FPRINTF_STDERR( "[chuck]: (see 'chuck --help' for more info)\n" );
-                    exit( 1 );
+                    errorMessage1 = "invalid arguments for '--adc:'";
+                    errorMessage2 = "(see 'chuck --help' for more info)";
+                    break;
                 }
             }
             else if( !strncmp(argv[i], "--adc", 5) )
@@ -718,9 +783,9 @@ bool go( int argc, const char ** argv )
                 else
                 {
                     // error
-                    CK_FPRINTF_STDERR( "[chuck]: invalid arguments for '--deprecate'...\n" );
-                    CK_FPRINTF_STDERR( "[chuck]: ... (looking for :stop, :warn, or :ignore)\n" );
-                    exit( 1 );
+                    errorMessage1 = "invalid arguments for '--deprecate:'";
+                    errorMessage2 = " |- looking for STOP, WARN, or IGNORE";
+                    break;
                 }
             }
             // (added 1.3.0.0)
@@ -733,9 +798,9 @@ bool go( int argc, const char ** argv )
                 else
                 {
                     // error
-                    CK_FPRINTF_STDERR( "[chuck]: invalid arguments for '--chugin-load'...\n" );
-                    CK_FPRINTF_STDERR( "[chuck]: ... (looking for :on or :off)\n" );
-                    exit( 1 );
+                    errorMessage1 = "invalid arguments for '--chugin-load:'";
+                    errorMessage2 = " |- looking for ON, OFF, or AUTO";
+                    break;
                 }
             }
             // (added 1.3.0.0)
@@ -755,11 +820,30 @@ bool go( int argc, const char ** argv )
             {
                 named_dls.push_back(argv[i]+sizeof("--chugin:")-1);
             }
-            // (added 1.3.0.0)
             else if( !strncmp(argv[i], "-g", sizeof("-g")-1) )
             {
                 named_dls.push_back(argv[i]+sizeof("-g")-1);
             }
+            // (added 1.5.0.5)
+            else if( !strncmp(argv[i], "--color:", sizeof("--color:")-1) )
+            {
+                // get the rest
+                string arg = tolower(argv[i]+sizeof("--color:")-1);
+                if( arg == "off" ) colorTerminal = false;
+                else if( arg == "on" || arg == "auto") colorTerminal = true;
+                else
+                {
+                    // error
+                    errorMessage1 = "invalid arguments for '--color:'...";
+                    errorMessage2 = "...(looking for ON, OFF, or AUTO)";
+                    break;
+                }
+            }
+            else if( tolower(argv[i]) == "--color" )
+                colorTerminal = TRUE;
+            else if( tolower(argv[i]) == "--no-color" )
+                colorTerminal = FALSE;
+            // (added 1.3.0.0)
             else if( !strcmp( argv[i], "--no-otf" ) )
             {
                 // don't use this new vm for otf commands (use the previous one)
@@ -767,20 +851,22 @@ bool go( int argc, const char ** argv )
             }
             else if( !strcmp( argv[i], "--probe" ) )
                 probe = TRUE;
+            else if( !strcmp( argv[i], "--chugin-probe" ) )
+                probe_chugs = TRUE;
             else if( !strcmp( argv[i], "--poop" ) )
                 uh();
             else if( !strcmp( argv[i], "--caution-to-the-wind" ) )
                 g_enable_system_cmd = TRUE;
+            else if( !strcmp( argv[i], "--disable-error-show-code" ) )
+                suppress_error_quote = TRUE;
             else if( !strcmp(argv[i], "--help") || !strcmp(argv[i], "-h")
                     || !strcmp(argv[i], "--about") )
             {
-                usage();
-                exit( 2 );
+                doAbout= TRUE;
             }
             else if( !strcmp( argv[i], "--version" ) )
             {
-                version();
-                exit( 2 );
+                doVersion = TRUE;
             }
             else
             {
@@ -799,9 +885,8 @@ bool go( int argc, const char ** argv )
                 if( is_otf ) exit( 1 );
                 
                 // done
-                CK_FPRINTF_STDERR( "[chuck]: invalid flag '%s'\n", argv[i] );
-                usage();
-                exit( 1 );
+                errorMessage1 = string("invalid flag '") + argv[i] + "' (see --help/-h)";
+                break;
             }
         }
         else // doesn't look like an argument
@@ -811,33 +896,84 @@ bool go( int argc, const char ** argv )
         }
     }
 
-    // log level (commented out; this is done below)
-    // EM_setlog( log_level );
+    // check if we output to TTY (teletype char at a time)
+    // if not disable color teriminal to avoid outputing
+    // ANSI escape codes to the output stream, which would
+    // show up in | and >
+    if( !ck_isatty() ) colorTerminal = false;
 
+    //------------------ CHUCK PRE-INITIALIZATION ---------------------
+    // instantiate a ChucK!
+    the_chuck = new ChucK();
+
+    // set the color terminal global setting
+    the_chuck->setParam( CHUCK_PARAM_TTY_COLOR, colorTerminal );
+
+    // set log level so things can start logging
+    the_chuck->setLogLevel( log_level );
+    //-----------------------------------------------------------------
+
+    // check for error message from command-line arguments
+    if( errorMessage1 != "" ) EM_error2( 0, errorMessage1.c_str() );
+    if( errorMessage2 != "" ) EM_error2( 0, errorMessage2.c_str() );
+    // if either
+    if( errorMessage1.length() || errorMessage2.length() )
+    {
+        // see if need to print usage
+        if( doAbout ) usage();
+        // bail for now
+        exit(1);
+    }
+
+    //-----------------------------------------------------------------
+    // about
+    //-----------------------------------------------------------------
+    if( doAbout )
+    {
+        usage();
+        exit( 2 );
+    }
+    //-----------------------------------------------------------------
+    // version
+    //-----------------------------------------------------------------
+    if( doVersion )
+    {
+        version();
+        exit( 2 );
+    }
+    //-----------------------------------------------------------------
     // probe
+    //-----------------------------------------------------------------
     if( probe )
     {
-        ChuckAudio::probe(audioDriver);
-        
+        // ensure log level is at least SYSTEM
+        if( the_chuck->getLogLevel() < CK_LOG_SYSTEM )
+            the_chuck->setLogLevel( CK_LOG_SYSTEM );
+
+        // probe default/selected audio driver
+        ChuckAudio::probe( audio_driver.c_str() );
+    
 #ifndef __DISABLE_MIDI__
+        // probe MIDI input and output devices
         EM_error2b( 0, "" );
         probeMidiIn();
         EM_error2b( 0, "" );
         probeMidiOut();
         EM_error2b( 0, "" );
 #endif  // __DISABLE_MIDI__
-        
+
+        // probe HID devices
         HidInManager::probeHidIn();
-        
-        // exit
+
+        // done probe
         exit( 0 );
     }
-    
+
     // set caution to wind
     ChucK::enableSystemCall = g_enable_system_cmd;
-    
+
     // check buffer size
-    buffer_size = ensurepow2( buffer_size );
+    buffer_size = ck_ensurepow2( buffer_size );
     // check mode and blocking
     if( !g_enable_realtime_audio && !block ) block = TRUE;
     // audio, boost 1.4.1.0 (ge) commented out
@@ -849,13 +985,13 @@ bool go( int argc, const char ** argv )
     g_do_watchdog = do_watchdog;
     // set adaptive size
     if( adaptive_size < 0 ) adaptive_size = buffer_size;
-    
-    if( !files && vm_halt && !g_enable_shell )
+
+    if( !files && vm_halt && !g_enable_shell && !probe_chugs )
     {
-        CK_FPRINTF_STDERR( "[chuck]: no input files... (try --help)\n" );
+        EM_error2( 0, "no input files... (try --help/-h)" );
         exit( 1 );
     }
-    
+
     // shell initialization without vm
     if( g_enable_shell && no_vm )
     {
@@ -867,7 +1003,7 @@ bool go( int argc, const char ** argv )
         // no vm is needed, just start running the shell now
         g_shell->run();
         // clean up
-        SAFE_DELETE( g_shell );
+        CK_SAFE_DELETE( g_shell );
         // done
         exit( 0 );
     }
@@ -880,10 +1016,10 @@ bool go( int argc, const char ** argv )
     }
     
     // find dac_name if appropriate (added 1.3.0.0)
-    if( dac_name.size() > 0 )
+    if( !probe_chugs && dac_name.size() > 0 )
     {
         // check with RtAudio
-        t_CKINT dev = ChuckAudio::device_named( audioDriver, dac_name, TRUE, FALSE );
+        t_CKINT dev = ChuckAudio::device_named( audio_driver.c_str(), dac_name, TRUE, FALSE );
         if( dev >= 0 )
         {
             dac = dev;
@@ -897,10 +1033,10 @@ bool go( int argc, const char ** argv )
     }
     
     // find adc_name if appropriate (added 1.3.0.0)
-    if( adc_name.size() > 0 )
+    if( !probe_chugs && adc_name.size() > 0 )
     {
         // check with RtAudio
-        t_CKINT dev = ChuckAudio::device_named( audioDriver, adc_name, FALSE, TRUE );
+        t_CKINT dev = ChuckAudio::device_named( audio_driver.c_str(), adc_name, FALSE, TRUE );
         if( dev >= 0 )
         {
             adc = dev;
@@ -914,19 +1050,38 @@ bool go( int argc, const char ** argv )
     }
 
 
-    //-------------------- VIRTUAL MACHINE SETUP (PART 1) -------------------------
-    // instantiate a ChucK!
-    the_chuck = new ChucK();
-    // set log level so things can start logging
-    the_chuck->setLogLevel( log_level );
+    //-------------------- VIRTUAL MACHINE SETUP (PART 1) ---------------------
+    // set chugins parameters
+    the_chuck->setParam( CHUCK_PARAM_CHUGIN_ENABLE, chugin_load );
+    the_chuck->setParam( CHUCK_PARAM_USER_CHUGINS, named_dls );
+    the_chuck->setParam( CHUCK_PARAM_USER_CHUGIN_DIRECTORIES, dl_search_path );
+
+    //-----------------------------------------------------------------
+    // probe chugins | 1.5.0.4 (ge) added
+    //-----------------------------------------------------------------
+    if( probe_chugs )
+    {
+        // ensure log level is at least SYSTEM
+        if( the_chuck->getLogLevel() < CK_LOG_SYSTEM )
+            the_chuck->setLogLevel( CK_LOG_SYSTEM );
+
+        // probe chugins (.chug and .ck modules)
+        // print/log what ChucK would load with current settings
+        the_chuck->probeChugins();
+
+        // done with probe
+        exit( 0 );
+    }
+
     // log
-    EM_log( CK_LOG_SYSTEM, "booting up ChucK instance..." );
+    EM_log( CK_LOG_SYSTEM, "initializing chuck..." );
     // push
     EM_pushlog();
 
-    //--------------------------- AUDIO I/O SETUP ---------------------------------
+
+    //--------------------------- AUDIO I/O SETUP -----------------------------
     // log
-    EM_log( CK_LOG_SYSTEM, "initializing audio subsystem..." );
+    EM_log( CK_LOG_SYSTEM, "initializing system settings..." );
     // push
     EM_pushlog();
     // log
@@ -940,7 +1095,7 @@ bool go( int argc, const char ** argv )
     {
         // initialize real-time audio
         t_CKBOOL retval = ChuckAudio::initialize( dac, adc, dac_chans, adc_chans,
-            srate, buffer_size, num_buffers, cb, (void *)the_chuck, force_srate, audioDriver );
+            srate, buffer_size, num_buffers, cb, (void *)the_chuck, force_srate, audio_driver.c_str() );
         // check return code
         if( !retval )
         {
@@ -960,6 +1115,7 @@ bool go( int argc, const char ** argv )
         dac = ChuckAudio::m_dac_n+1;
         adc_chans = ChuckAudio::m_num_channels_in;
         dac_chans = ChuckAudio::m_num_channels_out;
+        audio_driver = ChuckAudio::m_driver_name;
         adc_device_name = ChuckAudio::m_adc_name;
         dac_device_name = ChuckAudio::m_dac_name;
         srate = ChuckAudio::m_sample_rate;
@@ -971,7 +1127,7 @@ bool go( int argc, const char ** argv )
     EM_poplog();
 
 
-    //-------------------- VIRTUAL MACHINE SETUP (PART 2) -------------------------
+    //-------------------- VIRTUAL MACHINE SETUP (PART 2) ---------------------
     // set params (some, like sample rate, could have been updated during
     // real-time audio initialization, so we are setting actual params here)
     // set the actual sample rate; needed to map to in-language durations
@@ -986,11 +1142,10 @@ bool go( int argc, const char ** argv )
     the_chuck->setParam( CHUCK_PARAM_DUMP_INSTRUCTIONS, (t_CKINT)dump );
     the_chuck->setParam( CHUCK_PARAM_AUTO_DEPEND, (t_CKINT)auto_depend );
     the_chuck->setParam( CHUCK_PARAM_DEPRECATE_LEVEL, deprecate_level );
-    the_chuck->setParam( CHUCK_PARAM_CHUGIN_ENABLE, chugin_load );
-    the_chuck->setParam( CHUCK_PARAM_USER_CHUGINS, named_dls );
-    the_chuck->setParam( CHUCK_PARAM_USER_CHUGIN_DIRECTORIES, dl_search_path );
     // set hint, so internally can advise things like async data writes etc.
-    the_chuck->setParam( CHUCK_PARAM_HINT_IS_REALTIME_AUDIO, g_enable_realtime_audio );
+    the_chuck->setParam( CHUCK_PARAM_IS_REALTIME_AUDIO_HINT, g_enable_realtime_audio );
+    // enable or disable highlighting code on compiler error
+    the_chuck->setParam( CHUCK_PARAM_COMPILER_HIGHLIGHT_ON_ERROR, (t_CKINT)!suppress_error_quote );
 
     // initialize
     if( !the_chuck->init() )
@@ -999,10 +1154,13 @@ bool go( int argc, const char ** argv )
         exit( 1 );
     }
 
-    //------------------ PRINT VERIFIED AUDIO SETTING ---------------------------
+    // timestamp
+    EM_log( CK_LOG_SYSTEM, "local time: %s", timestamp_formatted().c_str() );
 
+
+    //------------------ PRINT VERIFIED AUDIO SETTING -------------------------
     // log
-    EM_log( CK_LOG_SYSTEM, "audio subsystem enabled..." );
+    EM_log( CK_LOG_SYSTEM, "initializing audio I/O..." );
     // push
     EM_pushlog();
     // log
@@ -1014,7 +1172,7 @@ bool go( int argc, const char ** argv )
     {
         EM_log( CK_LOG_SYSTEM, "num buffers: %ld", num_buffers );
         EM_log( CK_LOG_SYSTEM, "adaptive block processing: %ld", adaptive_size > 1 ? adaptive_size : 0 );
-        EM_log( CK_LOG_SYSTEM, "audio driver: %s", audioDriver ? audioDriver : "(unspecified)");
+        EM_log( CK_LOG_SYSTEM, "audio driver: %s", audio_driver != "" ? audio_driver.c_str() : "(unspecified)");
         EM_log( CK_LOG_SYSTEM, "adc:[%d] \"%s\"", adc, adc_device_name.c_str() );
         EM_log( CK_LOG_SYSTEM, "dac:[%d] \"%s\"", dac, dac_device_name.c_str() );
         // EM_log( CK_LOG_SYSTEM, "adc: %ld dac: %d", adc, dac );
@@ -1023,27 +1181,71 @@ bool go( int argc, const char ** argv )
     // pop
     EM_poplog();
 
-    //------------------------- HID RELATED SETUP --------------------------------
+
+    //------------------------- HID RELATED SETUP -----------------------------
 #ifndef __ALTER_HID__
     // pre-load hid
     if( load_hid ) HidInManager::init();
 #endif // __ALTER_HID__
 
     
-    //------------------------- CLI RELATED SETUP --------------------------------
+    //------------------------- CLI RELATED SETUP -----------------------------
     // catch SIGINT
     signal( SIGINT, signal_int );
-#ifndef __PLATFORM_WIN32__
+#ifndef __PLATFORM_WINDOWS__
     // catch SIGPIPE
     signal( SIGPIPE, signal_pipe );
 #endif
 
-    // version
-    EM_log( CK_LOG_SYSTEM, "ChucK version: %s", the_chuck->version() );
+    // boost priority
+    if( XThreadUtil::our_priority != 0x7fffffff )
+    {
+        // try
+        if( !XThreadUtil::set_priority( XThreadUtil::our_priority ) )
+        {
+            // error
+            CK_FPRINTF_STDERR( "[chuck]: error setting thread priority...\n" );
+            exit( 1 );
+        }
+    }
 
-    //------------------------- SOURCE COMPILATION --------------------------------
+    // -------------------------- SHELL SETUP ---------------------------------
+    // shell initialization
+    if( g_enable_shell )
+    {
+        // instantiate
+        g_shell = new Chuck_Shell;
+        // initialize
+        if( !init_shell( g_shell, new Chuck_Console(), the_chuck->vm() ) )
+            exit( 1 );
+
+        // flag
+        g_shutdown_shell = FALSE;
+
+        // NOTE: shell to be started later below
+    }
+
+    // version
+    EM_log( CK_LOG_SYSTEM,  "chuck version: %s", TC::green(the_chuck->version(), true).c_str() );
+
+    // pop log
+    EM_poplog();
+
+
+    //----------------------- ENTERING CHUCK RUNTIME --------------------------
     // log
-    EM_log( CK_LOG_SEVERE, "starting compilation..." );
+    EM_log( CK_LOG_SYSTEM, "starting chuck..." );
+
+    // start it!
+    the_chuck->start();
+
+    // push indent
+    EM_pushlog();
+
+
+    //------------------------- SOURCE COMPILATION ----------------------------
+    // log
+    EM_log( CK_LOG_SYSTEM, "starting compilation..." );
     // push indent
     EM_pushlog();
 
@@ -1076,57 +1278,24 @@ bool go( int argc, const char ** argv )
     // pop indent
     EM_poplog();
     
+    // syntax check only mode
     if( syntax_check_only )
+    {
+        // pop log
+        EM_poplog();
+        // log message
+        EM_log( CK_LOG_SYSTEM, "syntax checking mode complete; exiting..." );
+        // done
         return TRUE;
-
-    // boost priority
-    if( XThreadUtil::our_priority != 0x7fffffff )
-    {
-        // try
-        if( !XThreadUtil::set_priority( XThreadUtil::our_priority ) )
-        {
-            // error
-            CK_FPRINTF_STDERR( "[chuck]: error setting thread priority...\n" );
-            exit( 1 );
-        }
-    }
-    
-    
-    //----------------------- SHELL SETUP -----------------------------
-
-    // shell initialization
-    if( g_enable_shell )
-    {
-        // instantiate
-        g_shell = new Chuck_Shell;
-        // initialize
-        if( !init_shell( g_shell, new Chuck_Console(), the_chuck->vm() ) )
-            exit( 1 );
-
-        // flag
-        g_shutdown_shell = FALSE;
-
-        // start shell on separate thread | REFACTOR-2017: per-VM?!?
-#if !defined(__PLATFORM_WIN32__) || defined(__WINDOWS_PTHREAD__)
-        pthread_create( &g_tid_shell, NULL, shell_cb, g_shell );
-#else
-        g_tid_shell = CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)shell_cb, g_shell, 0, 0 );
-#endif
     }
 
-    // pop log
-    EM_poplog();
-    
-    //-------------------------- MAIN CHUCK LOOP!!! -----------------------------
-    // log
-    EM_log( CK_LOG_SYSTEM, "running main loop..." );
+    // debug print: print all vm objects
+    CK_VM_DEBUGGER( print_all_objects() );
+    // debug print: print vm objects stats
+    CK_VM_DEBUGGER( print_stats() );
 
-    // start it!
-    the_chuck->start();
-    
-    // push indent
-    EM_pushlog();
-    
+
+    //------------------------- STARTING AUDIO I/O ----------------------------
     // silent mode buffers
     SAMPLE * input = new SAMPLE[buffer_size*adc_chans];
     SAMPLE * output = new SAMPLE[buffer_size*dac_chans];
@@ -1142,14 +1311,29 @@ bool go( int argc, const char ** argv )
         // start real-time audio I/O
         ChuckAudio::start();
     }
+
     
+    //------------------------- MAIN CHUCK LOOP!!! ----------------------------
     // log
     EM_log( CK_LOG_SYSTEM, "starting virtual machine..." );
+    EM_log( CK_LOG_SYSTEM, "starting main loop..." );
 
     // pop indent
     EM_poplog();
 
-    // wait
+    // chuck shell
+    // (deferring as late as possible to minimize output conflict with log)
+    if( g_enable_shell )
+    {
+        // start shell on separate thread | REFACTOR-2017: per-VM?!?
+#if !defined(__PLATFORM_WINDOWS__) || defined(__WINDOWS_PTHREAD__)
+        pthread_create( &g_tid_shell, NULL, shell_cb, g_shell );
+#else
+        g_tid_shell = CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)shell_cb, g_shell, 0, 0 );
+#endif
+    }
+
+    // run loop
     while( the_chuck->vm_running() )
     {
         // real-time audio
@@ -1164,7 +1348,7 @@ bool go( int argc, const char ** argv )
             if( hook ) {
                 hook->m_hook( hook->m_bindle );
             } else {
-                usleep( 10000 );
+                ck_usleep( 10000 );
             }
         }
         else // silent mode
